@@ -109,7 +109,7 @@ use Time::HiRes ();
 my $opt_bsd_resource = eval "use BSD::Resource; 1;";
 
 use vars qw{$VERSION};
-$VERSION = "1.47";
+$VERSION = "1.48";
 
 use warnings;
 no  warnings qw(deprecated);
@@ -376,6 +376,9 @@ sub RunTimers {
         $to_run->[1]->($now);
     }
 
+    # close any sockets that our timers might've closed
+    CloseClosedSockets() if @ToClose;
+
     return $LoopTimeout unless @Timers;
 
     # convert time to an even number of milliseconds, adding 1
@@ -622,6 +625,22 @@ sub SetPostLoopCallback {
     }
 }
 
+sub CloseClosedSockets {
+    # now we can close sockets that wanted to close during our event processing.
+    # (we didn't want to close them during the loop, as we didn't want fd numbers
+    #  being reused and confused during the event loop)
+    while (my $sock = shift @ToClose) {
+        my $fd = fileno($sock);
+
+        # close the socket.  (not a Danga::Socket close)
+        $sock->close;
+
+        # and now we can finally remove the fd from the map.  see
+        # comment above in _cleanup.
+        delete $DescriptorMap{$fd};
+    }
+}
+
 # Internal function: run the post-event callback, send read events
 # for pushed-back data, and close pending connections.  returns 1
 # if event loop should continue, or 0 to shut it all down.
@@ -647,20 +666,7 @@ sub PostEventLoop {
         }
     }
 
-    # now we can close sockets that wanted to close during our event processing.
-    # (we didn't want to close them during the loop, as we didn't want fd numbers
-    #  being reused and confused during the event loop)
-    while (my $sock = shift @ToClose) {
-        my $fd = fileno($sock);
-
-        # close the socket.  (not a Danga::Socket close)
-        $sock->close;
-
-        # and now we can finally remove the fd from the map.  see
-        # comment above in _cleanup.
-        delete $DescriptorMap{$fd};
-    }
-
+    CloseClosedSockets();
 
     # by default we keep running, unless a postloop callback (either per-object
     # or global) cancels it
@@ -994,6 +1000,17 @@ sub push_back_read {
     $PushBackSet{$self->{fd}} = $self;
 }
 
+### METHOD: shift_back_read( $buf )
+### Shift back I<buf> (a scalar or scalarref) into the read stream
+### Use this instead of push_back_read() when you need to unread
+### something you just read.
+sub shift_back_read {
+    my Danga::Socket $self = shift;
+    my $buf = shift;
+    unshift @{$self->{read_push_back}}, ref $buf ? $buf : \$buf;
+    $PushBackSet{$self->{fd}} = $self;
+}
+
 ### METHOD: read( $bytecount )
 ### Read at most I<bytecount> bytes from the underlying handle; returns scalar
 ### ref on read, or undef on connection closed.
